import * as i0 from '@angular/core';
import { InjectionToken, inject, EventEmitter, Renderer2, booleanAttribute, numberAttribute, Component, ChangeDetectionStrategy, Input, Output, ViewChild, makeEnvironmentProviders } from '@angular/core';
import { generateQrCodeImage$, generateQrCodeCanvas$ } from 'dfts-qrcode';

function downloadQRCode(dataURL, filename = 'qrcode') {
    // split into two parts
    const parts = dataURL.split(';base64,');
    // hold the content type
    const imageType = parts[0].split(':')[1];
    // decode base64 string
    const decodedData = atob(parts[1]);
    // create unit8array of size same as row data length
    const uInt8Array = new Uint8Array(decodedData.length);
    // insert all character code into uint8array
    for (let i = 0; i < decodedData.length; ++i) {
        uInt8Array[i] = decodedData.charCodeAt(i);
    }
    // return blob image after conversion
    const blobData = new Blob([uInt8Array], { type: imageType });
    // saves as image
    const blob = new Blob([blobData], { type: 'image/png' });
    const url = URL.createObjectURL(blob);
    const link = document.createElement('a');
    link.href = url;
    link.download = filename;
    link.click();
}

const QRCODE_ALLOW_EMPTY_STRING = new InjectionToken('QRCODE_ALLOW_EMPTY_STRING', {
    factory: () => false,
});
const QRCODE_COLOR_DARK = new InjectionToken('QRCODE_COLOR_DARK', {
    factory: () => '#000000',
});
const QRCODE_COLOR_LIGHT = new InjectionToken('QRCODE_COLOR_LIGHT', {
    factory: () => '#ffffff',
});
const QRCODE_CSS_CLASS = new InjectionToken('QRCODE_CSS_CLASS', {
    factory: () => 'qrcode',
});
const QRCODE_ELEMENT_TYPE = new InjectionToken('QRCODE_ELEMENT_TYPE', {
    factory: () => 'img',
});
const QRCODE_ERROR_CORRECTION_LEVEL = new InjectionToken('QRCODE_ERROR_CORRECTION_LEVEL', {
    factory: () => 'L',
});
const QRCODE_MARGIN = new InjectionToken('QRCODE_MARGIN', {
    factory: () => 4,
});
const QRCODE_IMAGE_SRC = new InjectionToken('QRCODE_IMAGE_SRC', {
    factory: () => undefined,
});
const QRCODE_IMAGE_HEIGHT = new InjectionToken('QRCODE_IMAGE_HEIGHT', {
    factory: () => 40,
});
const QRCODE_IMAGE_WIDTH = new InjectionToken('QRCODE_IMAGE_WIDTH', {
    factory: () => 40,
});
const QRCODE_VERSION = new InjectionToken('QRCODE_VERSION', {
    factory: () => -1,
});
const QRCODE_SIZE = new InjectionToken('QRCODE_WIDTH', {
    factory: () => 5,
});

class QRCodeComponent {
    constructor() {
        this.allowEmptyString = inject(QRCODE_ALLOW_EMPTY_STRING);
        this.colorDark = inject(QRCODE_COLOR_DARK);
        this.colorLight = inject(QRCODE_COLOR_LIGHT);
        this.cssClass = inject(QRCODE_CSS_CLASS);
        this.elementType = inject(QRCODE_ELEMENT_TYPE);
        this.errorCorrectionLevel = inject(QRCODE_ERROR_CORRECTION_LEVEL);
        this.version = inject(QRCODE_VERSION);
        this.size = inject(QRCODE_SIZE);
        this.margin = inject(QRCODE_MARGIN);
        this.imageSrc = inject(QRCODE_IMAGE_SRC);
        this.imageHeight = inject(QRCODE_IMAGE_HEIGHT);
        this.imageWidth = inject(QRCODE_IMAGE_WIDTH);
        this.data = '';
        this.qrCodeDataUrl = new EventEmitter();
        this.renderer = inject(Renderer2);
    }
    ngOnInit() {
        this.createQRCode();
    }
    ngOnChanges() {
        this.createQRCode();
    }
    isValidQrCodeText(data) {
        return data !== null && data !== undefined && data.length > 0;
    }
    renderElement(element) {
        for (const node of this.qrcElement.nativeElement.childNodes) {
            this.renderer.removeChild(this.qrcElement.nativeElement, node);
        }
        this.renderer.appendChild(this.qrcElement.nativeElement, element);
    }
    createQRCode() {
        const validQrData = this.isValidQrCodeText(this.data);
        if (!this.allowEmptyString && !validQrData) {
            console.error('[dfx-qrcode] Field `data` is empty, set \'allowEmptyString="true"\' to overwrite this behaviour.');
            return;
        }
        else if (this.allowEmptyString && !validQrData) {
            this.data = ' ';
        }
        try {
            const config = {
                errorCorrectionLevel: this.errorCorrectionLevel,
                version: this.version,
                size: this.size,
                alt: this.alt,
                title: this.title,
                ariaLabel: this.ariaLabel,
                margin: this.margin,
                colors: {
                    colorLight: this.colorLight,
                    colorDark: this.colorDark,
                },
                image: {
                    src: this.imageSrc,
                    width: this.imageWidth,
                    height: this.imageHeight,
                },
            };
            switch (this.elementType) {
                case 'canvas': {
                    generateQrCodeCanvas$(this.data, config, this.renderer.createElement('canvas')).then((element) => {
                        this.renderElement(element);
                        this.qrCodeDataUrl.emit(element.toDataURL());
                    });
                    break;
                }
                case 'img': {
                    generateQrCodeImage$(this.data, config, this.renderer.createElement('canvas'), this.renderer.createElement('img')).then(({ image, dataUrl }) => {
                        this.renderElement(image);
                        this.qrCodeDataUrl.emit(dataUrl);
                    });
                    break;
                }
                default:
                    console.error(`[dfx-qrcode] Error: Unknown elementType "${this.elementType}"`);
            }
        }
        catch (e) {
            console.error('[dfx-qrcode] Error generating QR Code:', e);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.1", ngImport: i0, type: QRCodeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "16.1.0", version: "17.0.1", type: QRCodeComponent, isStandalone: true, selector: "qrcode", inputs: { allowEmptyString: ["allowEmptyString", "allowEmptyString", booleanAttribute], colorDark: "colorDark", colorLight: "colorLight", cssClass: "cssClass", elementType: "elementType", errorCorrectionLevel: "errorCorrectionLevel", version: ["version", "version", numberAttribute], size: ["size", "size", numberAttribute], margin: ["margin", "margin", numberAttribute], imageSrc: "imageSrc", imageHeight: ["imageHeight", "imageHeight", numberAttribute], imageWidth: ["imageWidth", "imageWidth", numberAttribute], alt: "alt", ariaLabel: "ariaLabel", title: "title", data: "data" }, outputs: { qrCodeDataUrl: "qrCodeDataUrl" }, viewQueries: [{ propertyName: "qrcElement", first: true, predicate: ["qrcElement"], descendants: true, static: true }], usesOnChanges: true, ngImport: i0, template: `<div #qrcElement [class]="cssClass"></div>`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.1", ngImport: i0, type: QRCodeComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'qrcode',
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    standalone: true,
                    template: `<div #qrcElement [class]="cssClass"></div>`,
                }]
        }], propDecorators: { allowEmptyString: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], colorDark: [{
                type: Input
            }], colorLight: [{
                type: Input
            }], cssClass: [{
                type: Input
            }], elementType: [{
                type: Input
            }], errorCorrectionLevel: [{
                type: Input
            }], version: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], size: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], margin: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], imageSrc: [{
                type: Input
            }], imageHeight: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], imageWidth: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], alt: [{
                type: Input
            }], ariaLabel: [{
                type: Input
            }], title: [{
                type: Input
            }], data: [{
                type: Input
            }], qrCodeDataUrl: [{
                type: Output
            }], qrcElement: [{
                type: ViewChild,
                args: ['qrcElement', { static: true }]
            }] } });

var QRCodeFeatureKind;
(function (QRCodeFeatureKind) {
    QRCodeFeatureKind[QRCodeFeatureKind["ALLOW_EMPTY_STRING"] = 0] = "ALLOW_EMPTY_STRING";
    QRCodeFeatureKind[QRCodeFeatureKind["COLOR_DARK"] = 1] = "COLOR_DARK";
    QRCodeFeatureKind[QRCodeFeatureKind["COLOR_LIGHT"] = 2] = "COLOR_LIGHT";
    QRCodeFeatureKind[QRCodeFeatureKind["CSS_CLASS"] = 3] = "CSS_CLASS";
    QRCodeFeatureKind[QRCodeFeatureKind["ELEMENT_TYPE"] = 4] = "ELEMENT_TYPE";
    QRCodeFeatureKind[QRCodeFeatureKind["ERROR_CORRECTION_LEVEL"] = 5] = "ERROR_CORRECTION_LEVEL";
    QRCodeFeatureKind[QRCodeFeatureKind["IMAGE"] = 6] = "IMAGE";
    QRCodeFeatureKind[QRCodeFeatureKind["MARGIN"] = 7] = "MARGIN";
    QRCodeFeatureKind[QRCodeFeatureKind["VERSION"] = 8] = "VERSION";
    QRCodeFeatureKind[QRCodeFeatureKind["SIZE"] = 9] = "SIZE";
})(QRCodeFeatureKind || (QRCodeFeatureKind = {}));

var QRCodeImageFeatureKind;
(function (QRCodeImageFeatureKind) {
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["SRC"] = 0] = "SRC";
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["WIDTH"] = 1] = "WIDTH";
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["HEIGHT"] = 2] = "HEIGHT";
})(QRCodeImageFeatureKind || (QRCodeImageFeatureKind = {}));

function provideQRCode(...features) {
    return makeEnvironmentProviders([features.map((it) => it.providers)]);
}
function withAllowEmptyString(allowEmptyString) {
    return {
        kind: QRCodeFeatureKind.ALLOW_EMPTY_STRING,
        providers: [{ provide: QRCODE_ALLOW_EMPTY_STRING, useValue: allowEmptyString }],
    };
}
function withColorDark(colorDark) {
    return {
        kind: QRCodeFeatureKind.COLOR_DARK,
        providers: [{ provide: QRCODE_COLOR_DARK, useValue: colorDark }],
    };
}
function withColorLight(colorLight) {
    return {
        kind: QRCodeFeatureKind.COLOR_LIGHT,
        providers: [{ provide: QRCODE_COLOR_LIGHT, useValue: colorLight }],
    };
}
function withCssClass(cssClass) {
    return {
        kind: QRCodeFeatureKind.CSS_CLASS,
        providers: [{ provide: QRCODE_CSS_CLASS, useValue: cssClass }],
    };
}
function withElementType(elementType) {
    return {
        kind: QRCodeFeatureKind.ELEMENT_TYPE,
        providers: [{ provide: QRCODE_ELEMENT_TYPE, useValue: elementType }],
    };
}
function withErrorCorrectionLevel(errorCorrectionLevel) {
    return {
        kind: QRCodeFeatureKind.ERROR_CORRECTION_LEVEL,
        providers: [
            {
                provide: QRCODE_ERROR_CORRECTION_LEVEL,
                useValue: errorCorrectionLevel,
            },
        ],
    };
}
function withMargin(margin) {
    return {
        kind: QRCodeFeatureKind.MARGIN,
        providers: [{ provide: QRCODE_MARGIN, useValue: margin }],
    };
}
function withImage(...features) {
    return {
        kind: QRCodeFeatureKind.IMAGE,
        providers: features.map((it) => it.provider),
    };
}
function withImageSrc(src) {
    return {
        kind: QRCodeImageFeatureKind.SRC,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: src },
    };
}
function withImageWidth(width) {
    return {
        kind: QRCodeImageFeatureKind.WIDTH,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: width },
    };
}
function withImageHeight(height) {
    return {
        kind: QRCodeImageFeatureKind.HEIGHT,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: height },
    };
}
function withVersion(version) {
    return {
        kind: QRCodeFeatureKind.VERSION,
        providers: [{ provide: QRCODE_VERSION, useValue: version }],
    };
}
function withSize(size) {
    return {
        kind: QRCodeFeatureKind.SIZE,
        providers: [{ provide: QRCODE_SIZE, useValue: size }],
    };
}

/**
 * Generated bundle index. Do not edit.
 */

export { QRCODE_ALLOW_EMPTY_STRING, QRCODE_COLOR_DARK, QRCODE_COLOR_LIGHT, QRCODE_CSS_CLASS, QRCODE_ELEMENT_TYPE, QRCODE_ERROR_CORRECTION_LEVEL, QRCODE_IMAGE_HEIGHT, QRCODE_IMAGE_SRC, QRCODE_IMAGE_WIDTH, QRCODE_MARGIN, QRCODE_SIZE, QRCODE_VERSION, QRCodeComponent, QRCodeFeatureKind, QRCodeImageFeatureKind, downloadQRCode, provideQRCode, withAllowEmptyString, withColorDark, withColorLight, withCssClass, withElementType, withErrorCorrectionLevel, withImage, withImageHeight, withImageSrc, withImageWidth, withMargin, withSize, withVersion };
//# sourceMappingURL=dfx-qrcode.mjs.map
